"""
Reviews service
"""

import logging
from typing import List

from portal.database.models import Review, ReviewItem, Submission, User
from portal.service.general import GeneralService

log = logging.getLogger(__name__)


class ReviewService(GeneralService):
    @property
    def _entity_klass(self):
        return Review

    def _set_data(self, entity, **data):
        allowed = []
        return self.update_entity(entity, data, allowed=allowed)

    def __call__(self, entity: Review = None, submission: Submission = None):
        super(ReviewService, self).__call__(entity=entity)
        self._submission = submission
        return self

    @property
    def review(self) -> Review:
        return self._entity or self.submission.review

    @property
    def submission(self):
        return self._submission or self._entity.submission

    def create_review_items(self, author: User, items: List[dict]) -> Review:
        """Creates review items for the review

        Args:
            author(User): Review's author
            items(List[dict]): Review items to append to the review

        Returns(Review): review instance with review items
        """
        for item in items:
            ReviewItem(user=author, review=self.review, file=item['file'], line=item['line'],
                       content=item['content'])
        log.info(f"Added review items {items} to review {self.review.id} "
                 f"for submission {self.submission.log_name} by {self.client_name}")
        self.write_entity(self.review)
        return self.review

    def create(self) -> Review:
        """Creates instance of the review

        Args:

        Returns(Review): Review instance
        """
        log.debug(f"[CREATE] Review for submission by "
                  f"{self.client_name}: {self.submission.log_name}")
        review = Review(submission=self.submission)
        self.write_entity(review)
        return review
