"""
Time module
"""

import datetime
import json

import pytz

UTC = pytz.UTC

# portal_timezone = datetime.timezone(offset=datetime.timedelta(hours=1), name='UTC+1')

portal_timezone = pytz.timezone('Europe/Prague')

DEFAULT_FORMAT = "%d-%m-%Y %H:%M:%S."


def current_time() -> datetime:
    return datetime.datetime.now(tz=portal_timezone)


def normalize_time(time) -> datetime:
    if hasattr(time, 'tzinfo') and time.tzinfo is not None:
        return time
    return portal_timezone.localize(time)


def strip_seconds(time: datetime) -> datetime:
    if not time:
        return time
    replaced = time.replace(second=0, microsecond=0)
    return normalize_time(replaced)


class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):  # pylint: disable=E0202
        if isinstance(o, datetime.datetime):
            return o.isoformat()

        return json.JSONEncoder.default(self, o)
