import os

import pytest

from management.data import DataManagement
from portal import create_app, db
from portal.database import ProjectConfig
from portal.logger import Logging
from tests.utils.ent_mocker import EntitiesMocker


@pytest.fixture(scope='function')
def app():
    os.environ["PORTAL_CONFIG_TYPE"] = 'test'
    flask_app = create_app(environment='test')
    from tests import init_db
    init_db(flask_app)
    with flask_app.app_context():
        DataManagement(flask_app, db).create_admin_user('admin', '789789')
        DataManagement(flask_app, db).init_data('test')
        yield flask_app
        db.session.remove()
        db.drop_all()


@pytest.fixture(scope='function')
def client(app):
    with app.test_client() as _client:
        yield _client


@pytest.fixture()
def rest_service(app):
    from portal.service.rest import RestService
    return RestService()


@pytest.fixture()
def ent_mocker(app):
    from tests.utils.ent_mocker import EntitiesMocker
    return EntitiesMocker(app, db=db)


@pytest.fixture()
def mocked_submission(ent_mocker: EntitiesMocker, rest_service):
    submission = ent_mocker.create_submission()
    rest_service.submissions.write_entity(submission)
    ent_mocker.create_submission_storage(submission=submission)
    project_config: ProjectConfig = submission.project.config
    project_config.test_files_commit_hash = 'some-random-hash'
    return submission


@pytest.fixture()
def created_course(rest_service):
    return rest_service.find.course('testcourse1')


@pytest.fixture()
def created_course2(rest_service):
    return rest_service.find.course('testcourse2')


@pytest.fixture()
def created_project(rest_service, created_course):
    return rest_service.find.project(created_course, 'hw01')


@pytest.fixture()
def created_role_student(rest_service, created_course):
    return rest_service.find.role(created_course, 'student')


@pytest.fixture()
def created_role_teacher(rest_service, created_course):
    return rest_service.find.role(created_course, 'teacher')


@pytest.fixture()
def created_group(rest_service, created_course):
    return rest_service.find.group(created_course, 'seminar01')


@pytest.fixture()
def created_student(rest_service):
    return rest_service.find.user('student1')


@pytest.fixture()
def created_admin(rest_service):
    return rest_service.find.user('admin')


@pytest.fixture()
def created_teacher(rest_service):
    return rest_service.find.user('teacher1')


@pytest.fixture()
def created_worker(rest_service):
    return rest_service.find.worker('executor')
