from pathlib import Path

import pytest

from portal.database import ProjectConfig
from tests.utils.ent_mocker import EntitiesMocker
from . import rest_tools


def expected(fpath):
    return f'This is a test file: {fpath}'


def assert_path(data: dict, path: str):
    current = data
    path = Path(path)
    for part in path.parts:
        current = current.get(part)

    assert Path(current) == path


def test_submission_sources_are_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/sources?path=src/main.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 200
    assert response.data
    assert response.data.decode('utf-8') == expected('src/main.c')


def test_submission_sources_are_not_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/sources?path=nonexisting.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 404


def test_submission_test_files_are_not_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/test_files?path=nonexisting.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 404


def test_submission_results_are_not_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/results?path=nonexisting.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 404


def test_submission_results_are_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/results?path=student.json'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 200
    assert response.data
    assert response.data.decode('utf-8') == expected('student.json')


def test_submission_test_files_are_available(client, mocked_submission):
    s = mocked_submission
    url = f'/submissions/{s.id}/files/test_files?path=test_main.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 200
    assert response.data
    assert response.data.decode('utf-8') == expected('test_main.c')


def test_project_test_files_are_available(client, mocked_submission):
    s = mocked_submission
    url = f'/courses/{s.course.id}/projects/{s.project.id}/files?path=test_main.c'
    response = rest_tools.make_request(client, url)
    assert response.status_code == 200
    assert response.data
    assert response.data.decode('utf-8') == expected('test_main.c')


def test_submission_sources_tree(client, mocked_submission):
    s = mocked_submission
    response = rest_tools.make_request(client, f'/submissions/{s.id}/files/sources/tree')
    assert response.status_code == 200
    assert response.data
    data = rest_tools.extract_data(response=response)
    assert_path(data, 'src/main.c')
    assert_path(data, 'src/foo.c')


def test_submission_test_files_tree(client, mocked_submission):
    s = mocked_submission
    response = rest_tools.make_request(client, f'/submissions/{s.id}/files/test_files/tree')
    assert response.status_code == 200
    assert response.data
    data = rest_tools.extract_data(response=response)
    assert_path(data, 'test_main.c')


def test_submission_results_tree(client, mocked_submission):
    s = mocked_submission
    response = rest_tools.make_request(client, f'/submissions/{s.id}/files/results/tree')
    assert response.status_code == 200
    assert response.data
    data = rest_tools.extract_data(response)

    assert_path(data, 'results.json')
    assert_path(data, 'student.json')
